"pac" <- function(data,dis,rho,nperm=10)
{
  # void simplepac(int *data, int *ss, int *sites, double *d, double *rho, int *nrho,int *nperm,double *ll) {
 res <- 
  .C("simplepac"
     ,as.integer(t(data))
     ,as.integer(nrow(data))
     ,as.integer(ncol(data))
     ,as.double(dis)
     ,as.double(rho)
     ,as.integer(length(rho))
     ,as.integer(nperm)
     ,as.double(numeric(length(rho)),PACKAGE="pac")
     )[[8]]
 return(res);
}

#------------------------------------------------------------------------
"mlpac" <- function(xx,dis,interval=c(0,10),nperm=10,seed=1)
{
  if (inherits(xx,"GenomicHaplotype")) {
    if (!missing(dis))
      stop("do not use dis with xx of class GenomicHaplotype")
    yy <- as.integer(t(xx$d))
    ss <- as.integer(nrow(xx$d))
    loci <- as.integer(ncol(xx$d))
    disyy <- as.double(diff(xx$pos))
  } else {
    yy <- as.integer(t(xx))
    ss <- as.integer(nrow(xx))
    loci <- as.integer(ncol(xx))
    disyy <- as.double(dis)
  }
  if (seed==0) currseed <- .Random.seed

  if (ss<=1)
    stop("need at least two rows of data to estimate rho")

  if (loci<=1)
    stop("need at least two coluns of data to estimate rho")
  
  f <- function(r) {
    if (seed==0) .Random.seed <<- currseed
    else set.seed(seed);
    return(
      .C("simplepac1"
         ,yy,ss,loci,disyy
         ,as.double(r)
         ,as.integer(nperm)
         ,as.double(numeric(1))
         ,PACKAGE="pac"
         )[[7]]
           )
  }
  return(optimise(f,interval=interval,maximum=TRUE))
}
#------------------------------------------------------------------------
"recombinations" <- function(xx,dis,rho,nperm=10,interval=c(0,200))
  {
    if (inherits(xx,"GenomicHaplotype")) {
      if(missing(rho))
        rho <- mlpac(xx,seed=0);
      if (!missing(dis))
        stop("do not use dis with xx of class sima")
      yy <- as.integer(t(xx$d))
      ss <- as.integer(nrow(xx$d))
      loci <- as.integer(ncol(xx$d))
      disyy <- as.double(diff(xx$pos))
    } else {
      if (missing(dis))
        stop("Need to provide distances")
      yy <- as.integer(t(xx))
      ss <- as.integer(nrow(xx))
      loci <- as.integer(ncol(xx))
      disyy <- as.double(dis)
      if(missing(rho))
        rho <- mlpac(xx,dis,seed=0);
    }
    
    return(.C("recombinations"
              ,yy,ss,loci,disyy
              ,as.double(rho)
              ,as.integer(nperm)
              ,as.double(numeric(loci-1)),PACKAGE="pac"
              )[[7]])           
  }
#------------------------------------------------------------------------
"path" <- function(xx,dis,rho,interval=c(0,200))
  {
    if (inherits(xx,"GenomicHaplotype")) {
      if(missing(rho))
        rho <- mlpac(xx,seed=0);
      if (!missing(dis))
        stop("do not use dis with xx of class sima")
      yy <- as.integer(t(xx$d))
      ss <- as.integer(nrow(xx$d))
      loci <- as.integer(ncol(xx$d))
      disyy <- as.double(diff(xx$pos))
    } else {
      if (missing(dis))
        stop("Need to provide distances")
      yy <- as.integer(t(xx))
      ss <- as.integer(nrow(xx))
      loci <- as.integer(ncol(xx))
      disyy <- as.double(dis)
      if(missing(rho))
        rho <- mlpac(xx,dis,seed=0);
    }
    
    samplepath <- .C("samplepath"
                     ,yy,ss,loci,disyy
                     ,as.double(rho)
                     ,as.integer(numeric(ss*loci))
                     ,as.integer(numeric(ss))
                     ,as.double(1)
                     ,PACKAGE="pac"
              )
    list(
         path=matrix(samplepath[[6]]+1,nrow=ss,byrow=TRUE)
         ,order=samplepath[[7]]+1
         ,ll=samplepath[[8]]
         )
  }
#------------------------------------------------------------------------
"treefrompath" <- function(path,position)
  {
    ## convert the path to a tree?
    
    
  }
#------------------------------------------------------------------------
"sharedsection" <- function(xx,dis,rho,pos,interval=c(0,200),nperm=10)
  {
    ## calculate the lengths of sections shared with
    ## the chromosome the path is copied from (or copied to)
    if (inherits(xx,"GenomicHaplotype")) {
      if(missing(rho))
        rho <- mlpac(xx,seed=0);
      if (!missing(dis))
        stop("do not use dis with xx of class sima")
      yy <- as.integer(t(xx$d))
      ss <- as.integer(nrow(xx$d))
      loci <- as.integer(ncol(xx$d))
      disyy <- as.double(diff(xx$pos))
    } else {
      if (missing(dis))
        stop("Need to provide distances")
      yy <- as.integer(t(xx))
      ss <- as.integer(nrow(xx))
      loci <- as.integer(ncol(xx))
      disyy <- as.double(dis)
      if(missing(rho))
        rho <- mlpac(xx,dis,seed=0);
    }
    if (missing(pos))
      pos <- as.integer(loci/2)
    else pos <- as.integer(pos)

   sharedsection <- .C("sharedancestry"
                       ,yy,ss,loci,disyy
                       ,pos
                       ,as.double(rho)
                       ,as.integer(nperm)
                       ,as.double(numeric(ss*loci))
                       ,PACKAGE="pac"
                       )
  
    matrix(sharedsection[[8]],nrow=ss,byrow=TRUE)  
  }
  
