\name{recombinations}
\alias{recombinations}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Get an estimate of the observed number of recombinations from
  the pac model}
\description{
   A function to get an estimate of the observed number of recombinations from
  the pac model by forward backward simulation of pacs by the method of
  Li & Stephens (Genetics 2003).  Initially estimates the maximum
  likelihood value for rho (if this is nopt provided). A scaled mean
  number of recombinations is returned.  The scaling is based on the
  likelihood of the particular ordering used.}
\usage{ recombinations(data, dis, interval =
  c(0, 10), nperm = 10, seed = 0)
} %- maybe also 'usage' for otherobjects documented here.
\arguments{
  \item{data}{ A matrix of veriation data (0's and 1's). }
  \item{dis}{ The distance between sites. }
  \item{rho}{ The value of rho to used (if not set uses maximum likelihood).}
  \item{nperm}{ The number of permutations}
}
\value{
  Returns a vector of length sites-1 with the (scaled) mean number of
  recombinations between pairs of sites.  The scaling is based on the
  likelihood of the particular ordering used.
}
\references{ \url{http://www.mas.ncl.ac.uk/~nijw}
  
  Li, N & Stephens M. 2003. Modeling linkage disequilibrium and
  identfying recombination hotspots using single-nucleotide polymorphism
  data. Genetics. 165, 2213-2233
}
\author{ Ian Wilson <\email{I.J.Wilson@ncl.ac.uk}>}

\seealso{\code{\link{pac}}, \code{\link{mlpac}}}
\examples{
data(sim1) ## sim1 is provided by the genomic package
midp <- (sim1$pos[-1]+sim1$pos[-length(sim1$pos)])/2
## get two estimates of number of recombinations per site
r1 <- recombinations(sim1$d,diff(sim1$pos),nperm=50)/diff(sim1$pos)
r2 <- recombinations(sim1$d,diff(sim1$pos),nperm=50)/diff(sim1$pos)
plot(midp,r1 ,type="l",main="Average recombinations")
lines(midp,r2,col="red")
}
\keyword{optimize }

