#ifndef UTILITYFUNCTIONALS_H_
#define UTILITYFUNCTIONALS_H_

/* template <typename T> */
/* class cumsum: public unary_function<T,T> { */
/* public: */
/*     cumsum():sum(0){} */
/*     T& operator()(const T &item) { */
/* 	sum += T; */
/* 	return sum; */
/*     } */
/* private: */
/*     T sum; */
/* }; */
#include <vector>
#include <algorithm>
#include <numeric>
#include <cmath>

template <typename T>
class cumsum {
public:
    cumsum():sum(0){}
    T& operator()(const T &item) {
	sum += item;
	return sum;
    }
private:
    T sum;
};
//// from Meyers, Effective STL p38
struct DeleteObject {
    template <typename T> 
    void operator()( const T* ptr) const 
	{
	    delete ptr;
	}
};

/** A class for set comparators   */
template<class T>
struct less {
  bool operator()(T x, T y) { return x<y; }
};
template<class T>
struct moreorequal {
  bool operator()(T x, T y) { return !(x<y); }
};


template <typename T>
T rescale(  T *a, int count) 
{
  T sm = std::accumulate(a,a+count,0.0);
  sm /= (double)count;
  for (int i=0;i<count;i++) a[i]/=sm;
  return log(sm);
}

// only written for character variables
template <typename ITOR>
std::vector<int> tabulate(ITOR first, ITOR last)
{
  std::vector<int> b(*(std::max_element(first,last))+1);
  ITOR i=first;
  while (i!=last) {
     b[*i]+=1;
     i++;
  }
  return b;
}

/// class that simulates a factorial function- but includes a lookup table.
class lfactorl {
public:
  lfactorl() {
    for (int i=0;i<32;i++) a[i]=0.0;
  }
  double operator()(int n) {
    if (n<=1) return 0.0;
    if (n<=31) return (a[n]>0.0) ? a[n] : (a[n]=lgamma((double)(n+1)));	
    else return lgamma((double)(n+1));
  };
 private:
  double a[32];
};

class success_rate {
 public:
  success_rate() {
    success=trials=0.0;
  }
  void operator++() {
    success+=1.0;
    trials +=1.0;
  }
  void operator--() {
    trials+=1.0;
  }
  double operator()() {
    return success/trials;
  }
  void clear() {
    success=0.0;
    trials=0.0;
  }
 private:
  double success;
  double trials;
};


#endif
